@@ -3,7 +3,7 @@
 #' This function takes a character vector and makes edits and merges values
 #' that are approximately equivalent yet not identical. It clusters values
 #' based on the key collision method, described here
-#' \url{https://github.com/OpenRefine/OpenRefine/wiki/Clustering-In-Depth}.
+#' \url{https://docs.openrefine.org/next/technical-reference/clustering-in-depth/}.
 #'
 #' @param vect Character vector, items to be potentially clustered and merged.
 #' @param ignore_strings Character vector, these strings will be ignored during
@@ -45,7 +45,9 @@ key_collision_merge <- function(vect, ignore_strings = NULL, bus_suffix = TRUE,
 
   # If ignore_strings is not NULL, make all values lower case then get uniques.
   if (!is.null(ignore_strings)) {
-    ignore_strings <- unique(tolower(ignore_strings[!is.na(ignore_strings)]))
+    ignore_strings <- unique(
+      cpp_tolower(ignore_strings[!is.na(ignore_strings)])
+    )
   }
 
   # Get vector of key values. If dict is not NULL, get vector of key values
