@@ -6,7 +6,7 @@ using namespace Rcpp;
 // Wrapper for the two KC merge functions (one with a data dict, one without).
 // [[Rcpp::export]]
 CharacterVector merge_KC_clusters(const CharacterVector &vect,
-                                  const CharacterVector &keys_vect,
+                                  CharacterVector &keys_vect,
                                   const CharacterVector &dict,
                                   const CharacterVector &keys_dict) {
   if(CharacterVector::is_na(dict[0])) {
@@ -35,28 +35,28 @@ CharacterVector merge_KC_clusters(const CharacterVector &vect,
 // passed to func "key_collision_merge".
 CharacterVector merge_KC_clusters_no_dict(const CharacterVector &clusters,
                                           const CharacterVector &vect,
-                                          const CharacterVector &keys_vect) {
+                                          CharacterVector &keys_vect) {
   // Create copy of vect to use as the output vector.
   CharacterVector output = clone(vect);
 
   // Create unordered_map, using clusters as keys, values will be the indices
   // of each cluster in keys_vect.
-  std::vector<std::string> cl = as<std::vector<std::string> >(clusters);
-  refinr_map keys_vect_map = create_map(keys_vect, cl);
+  refinr_map keys_vect_map = create_map(keys_vect, clusters);
 
   // Initialize variables used in the loop below.
   std::vector<int> curr_idx;
   int curr_idx_len;
-  String most_freq_string;
+  freq_string mfs;
+
+  // refinr_map uses pointers to CHARSXP SEXP as keys. Get pointers to the
+  // SEXP clusters, to iterate over in the loop below.
+  SEXP* ptr = get_string_ptr(clusters);
 
   // Iterate over clusters, make mass edits to output.
-  std::vector<std::string>::iterator clust_end = cl.end();
-  std::vector<std::string>::iterator iter;
-
-  for(iter = cl.begin(); iter != clust_end; ++iter) {
+  for(unsigned int j = 0; j < clusters.size(); ++j) {
     // Create subset of vect using the indices from keys_vect_map that
     // correspond to the current cluster iteration.
-    curr_idx = keys_vect_map[*iter];
+    curr_idx = keys_vect_map[ptr[j]];
     curr_idx_len = curr_idx.size();
     CharacterVector curr_vect(curr_idx_len);
     for(int i = 0; i < curr_idx_len; ++i) {
@@ -64,11 +64,11 @@ CharacterVector merge_KC_clusters_no_dict(const CharacterVector &clusters,
     }
 
     // Get the string that appears most often in curr_vect.
-    most_freq_string = most_freq_str(curr_vect);
+    most_freq_str(curr_vect, mfs);
 
     // For each index in curr_idx, edit output to be equal to most_freq_string.
     for(int n = 0; n < curr_idx_len; ++n) {
-      output[curr_idx[n]] = most_freq_string;
+      output[curr_idx[n]] = mfs.mf_str;
     }
   }
 
@@ -80,37 +80,34 @@ CharacterVector merge_KC_clusters_no_dict(const CharacterVector &clusters,
 // passed to func "key_collision_merge".
 CharacterVector merge_KC_clusters_dict(const CharacterVector &clusters,
                                        const CharacterVector &vect,
-                                       const CharacterVector &keys_vect,
+                                       CharacterVector &keys_vect,
                                        const CharacterVector &dict,
                                        const CharacterVector &keys_dict) {
   // Create copy of vect to use as the output vector.
   CharacterVector output = clone(vect);
 
   // Create two unordered_maps, both using clusters as keys. Values for
-  // keys_vect_map will be the indices of each cluster in keys_vect. Values for
-  // keys_dict_map will be the indices of each cluster in keys_dict.
-  std::vector<std::string> cl = as<std::vector<std::string> >(clusters);
-  refinr_map keys_vect_map = create_map(keys_vect, cl);
-  refinr_map keys_dict_map = create_map(keys_dict, cl);
+  // Values for keys_vect_map will be the indices of each cluster in keys_vect.
+  // Values for keys_dict_map will be the indices of each cluster in keys_dict.
+  refinr_map keys_vect_map = create_map(keys_vect, clusters);
+  refinr_map keys_dict_map = create_map(keys_dict, clusters);
 
   // Initialize variables used in the loop below.
-  std::string curr_clust;
   std::vector<int> curr_vect_idx;
   int curr_vect_len;
   std::vector<int> curr_dict_idx;
   int curr_dict_len;
   bool not_in_dict;
-  String most_freq_string;
+  freq_string mfs;
+
+  // refinr_map uses pointers to CHARSXP SEXP as keys. Get pointers to the
+  // SEXP clusters, to iterate over in the loop below.
+  SEXP* ptr = get_string_ptr(clusters);
 
   // Iterate over clusters, make mass edits to output.
-  std::vector<std::string>::iterator clust_end = cl.end();
-  std::vector<std::string>::iterator iter;
-
-  for(iter = cl.begin(); iter != clust_end; ++iter) {
-    curr_clust = *iter;
-
+  for(unsigned int j = 0; j < clusters.size(); ++j) {
     // Create subset of vect using the indices that correspond to curr_clust.
-    curr_vect_idx = keys_vect_map[curr_clust];
+    curr_vect_idx = keys_vect_map[ptr[j]];
     curr_vect_len = curr_vect_idx.size();
     CharacterVector curr_vect(curr_vect_len);
     for(int i = 0; i < curr_vect_len; ++i) {
@@ -119,7 +116,7 @@ CharacterVector merge_KC_clusters_dict(const CharacterVector &clusters,
 
     // Check to see if curr_clust appears in keys_dict_map. If it does, create
     // subset of dict using the indices that correspond to curr_clust.
-    curr_dict_idx = keys_dict_map[curr_clust];
+    curr_dict_idx = keys_dict_map[ptr[j]];
     curr_dict_len = curr_dict_idx.size();
     CharacterVector curr_dict(curr_dict_len);
     not_in_dict = true;
@@ -134,19 +131,19 @@ CharacterVector merge_KC_clusters_dict(const CharacterVector &clusters,
     // most_freq_string from curr_dict. Otherwise get most_freq_string from
     // curr_vect.
     if(not_in_dict) {
-      most_freq_string = most_freq_str(curr_vect);
+      most_freq_str(curr_vect, mfs);
     } else {
       if(curr_dict_len == 1) {
-        most_freq_string = curr_dict[0];
+        mfs.mf_str = curr_dict[0];
       } else {
-        most_freq_string = most_freq_str(curr_dict);
+        most_freq_str(curr_dict, mfs);
       }
     }
 
     // For each index in curr_vect_idx, edit output to be equal to
     // most_freq_string.
     for(int n = 0; n < curr_vect_len; ++n) {
-      output[curr_vect_idx[n]] = most_freq_string;
+      output[curr_vect_idx[n]] = mfs.mf_str;
     }
   }
 
